#!/usr/bin/env bash

datadir="../../production_ab"
outdir="/data02b/photoz/pdr1/pdf"
outhost="har-fs02"

pdr1_ephor_ab_udeep_sxds=(
  8523 8524 8765 8766
)
pdr1_ephor_ab_udeep_cosmos=(
  9570 9571 9812 9813 9814 10054 10055
)
pdr1_ephor_ab_deep_xmmlss=(
  8282 8283 8284 8523 8524 8525 8765 8766 8767
)
pdr1_ephor_ab_deep_deep23=(
  9220 9221 9462 9463 9464 9465 9706 9707 9708
)
pdr1_ephor_ab_deep_cosmos=(
  9569 9570 9571 9812 9813 9814 10054 10055 10056
)
pdr1_ephor_ab_deep_elaisn1=(
  16984 16985 17129 17130 17131 17270 17271 17272 17406 17407
)
pdr1_ephor_ab_wide_xmmlss=(
  8279 8280 8281 8282 8283 8284 8285 8520 8521 8522
  8523 8524 8525 8526 8762 8763 8764 8765 8766 8767
  8768
)
pdr1_ephor_ab_wide_wide12h=(
  9346 9347 9348 9349 9589 9590 9591 9592
)
pdr1_ephor_ab_wide_gama15h=(
  9370 9371 9372 9373 9374 9375 9613 9614 9615 9616
  9617 9618
)
pdr1_ephor_ab_wide_gama09h=(
  9314 9315 9316 9317 9318 9557 9558 9559 9560 9561
  9562 9800 9801 9802 9803 9804
)
pdr1_ephor_ab_wide_vvds=(
  9450 9451 9452 9453 9454 9455 9456 9693 9694 9695
  9696 9697 9698 9699 9935 9936 9937 9938 9939 9940
  9941
)
pdr1_ephor_ab_wide_hectomap=(
  15830 15831 15832 15833 16008 16009 16010 16011
)
pdr1_ephor_ab_wide_aegis=(
  16821 16822 16972 16973
)

pdr1_udeep_fields=(
  pdr1_ephor_ab_udeep_sxds
  pdr1_ephor_ab_udeep_cosmos
)

pdr1_deep_fields=(
  pdr1_ephor_ab_deep_xmmlss
  pdr1_ephor_ab_deep_deep23
  pdr1_ephor_ab_deep_cosmos
  pdr1_ephor_ab_deep_elaisn1
)

pdr1_wide_fields=(
  pdr1_ephor_ab_wide_xmmlss
  pdr1_ephor_ab_wide_wide12h
  pdr1_ephor_ab_wide_gama15h
  pdr1_ephor_ab_wide_wide01h
  pdr1_ephor_ab_wide_vvds
  pdr1_ephor_ab_wide_gama09h
  pdr1_ephor_ab_wide_hectomap
  pdr1_ephor_ab_wide_aegis
)

oldIFS="$IFS"

#
# Check that the tract lists above are complete
#

for release in pdr1_{udeep,deep}
do
  all_tracts=()
  for field in $(eval "echo \${${release}_fields[@]}")
  do
    eval "all_tracts+=(\${$field[@]})"
  done

  IFS=","
  tractlist=",${all_tracts[*]},"
  IFS="$oldIFS"

  prefix="$datadir/pdf-$release"
  for path in $prefix-*[0-9].fits
  do
    tract=${path:${#prefix}+1:${#path}-${#prefix}-6}
    if [[ "$tractlist" != *,$tract,* ]]
    then
      echo "File cannot be classified as any field: $path" 1>&2
      exit 1
    fi
  done
done

#
# Make archives
#

pushd "$datadir"

commands=()

for release in pdr1_{udeep,deep}
do
  nfiles=0
  for field in $(eval "echo \${${release}_fields[@]}")
  do
    IFS=","
    eval "tractlist=\",\${$field[*]},\""
    IFS="$oldIFS"

    filelist=()

    prefix="pdf-$release"
    for path in $prefix-*[0-9].fits
    do
      tract=${path:${#prefix}+1:${#path}-${#prefix}-6}
      if [[ "$tractlist" = *,$tract,* ]]
      then
        filelist+=($path)
      fi
    done

    commands+=("tar --owner=0 --group=0 --mode=644 -cf - ${filelist[*]} | xz -9 | ssh $outhost \"cat > $outdir/$field.tar.xz\"")
    nfiles=$((nfiles + ${#filelist[@]}))
  done

  all_files=($prefix-*[0-9].fits)
  if [[ nfiles -ne ${#all_files[@]} ]]
  then
    echo "Mismatch in # of files archived and those existing: $release" 1>&2
    exit 1
  fi
done

for cmd in "${commands[@]}"
do
  echo "$cmd"
done | parallel -j 8

popd
