import numpy
import psycopg2

import os
import sys

def main():
    outdir, = sys.argv[1:]

    db = psycopg2.connect(
        database = "pdr1",
    )

    cursor = db.cursor()

    dtype = numpy.dtype([
        ("object_id"            , numpy.int64),
        ("gparent_flux_convolved_2_1"     , numpy.float64),
        ("rparent_flux_convolved_2_1"     , numpy.float64),
        ("iparent_flux_convolved_2_1"     , numpy.float64),
        ("zparent_flux_convolved_2_1"     , numpy.float64),
        ("yparent_flux_convolved_2_1"     , numpy.float64),
    ])

    for schema in ["pdr1_udeep", "pdr1_deep", "pdr1_wide"]:
        cursor.execute("""
        SET search_path TO %(schema)s,public
        """, locals()
        )

        for tract in xrange(20000):
            print "{schema} {tract}...".format(**locals())

            idstart = tract     * (2**42)
            idend   = (tract+1) * (2**42) - 1
            cursor.execute("""
            WITH tmp AS(SELECT
                object_id                                                      AS object_id
            ,   gparent_flux_convolved_2_1     * c_exp(0.92103403719761845::float8*a_g)  AS gparent_flux_convolved_2_1
            ,   rparent_flux_convolved_2_1     * c_exp(0.92103403719761845::float8*a_r)  AS rparent_flux_convolved_2_1
            ,   iparent_flux_convolved_2_1     * c_exp(0.92103403719761845::float8*a_i)  AS iparent_flux_convolved_2_1
            ,   zparent_flux_convolved_2_1     * c_exp(0.92103403719761845::float8*a_z)  AS zparent_flux_convolved_2_1
            ,   yparent_flux_convolved_2_1     * c_exp(0.92103403719761845::float8*a_y)  AS yparent_flux_convolved_2_1
            FROM
                forced join afterburner using (object_id)
            WHERE
                (object_id BETWEEN %(idstart)s AND %(idend)s)
            )
            SELECT
                *
            FROM
                tmp
            WHERE
                    c_isfinite(gparent_flux_convolved_2_1    ) != 0
                AND c_isfinite(rparent_flux_convolved_2_1    ) != 0
                AND c_isfinite(iparent_flux_convolved_2_1    ) != 0
                AND c_isfinite(zparent_flux_convolved_2_1    ) != 0
                AND c_isfinite(yparent_flux_convolved_2_1    ) != 0
            """, locals()
            )

            catalog = numpy.array(cursor.fetchall(), dtype=dtype)
            if len(catalog) > 0:
                numpy.save(os.path.join(outdir, "input-{schema}-{tract}.npy".format(**locals())), catalog)

main()
