#!/usr/bin/env bash

# Copyright (c) 2016-2017 Sogo Mineo
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


# This program will update the copyright year
# at the beginning of each file in the working directory.
# For example, from
#   Copyright (c) 2016-2017  Jane Doe
# to
#   Copyright (c) 2016-2018  Jane Doe
# if it is 2018 when this script is executed.

author="Sogo Mineo"
year="$(date +"%Y")"

main()
(
  find . \( -name ".git" -prune -false \) -o -type f | while read filename
  do
    if  [[ "$(file "${filename}")" = *text* ]] &&
        grep -q "Copyright.*${author}" "${filename}"
    then
      python -c "${py_bumpup}" "${author}" "${year}" "${filename}"
    fi
  done
)

py_bumpup=$(cat << 'EOF'
import os
import re
import sys

author, year, filename = sys.argv[1:]
year = int(year)

def bumpup(match):
  prefix = match.group(1) or ""
  start  = int(match.group(2) or 0)
  end    = year

  if start == end:
    return prefix + str(start)
  else:
    return prefix + str(start) + "-" + str(end)

def main():
  with open(filename, "r") as f:
    input = f.read()
    output = re.compile(r"(Copyright *\(C\) *)([0-9]+)(-[0-9]+)?(?= *" + author + ")", flags=re.I).sub(bumpup, input)
  if input != output:
    print("Modifying {}".format(filename))
    mode = os.stat(filename).st_mode
    os.rename(filename, filename+"~")
    with open(filename, "w") as f:
      f.write(output)
      os.fchmod(f.fileno(), mode)

main()
EOF
)

main "$@"
