#!/usr/bin/env python

# Copyright (c) 2016-2017 Sogo Mineo
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from lib import hsc_stdio

import os


def startup():
    import argparse

    parser = argparse.ArgumentParser(fromfile_prefix_chars='@',
        description="""
            Redshifts are estimated from pdf
            such that a loss function is minimized.
        """
    )
    parser.add_argument('input',
        help="""
            Input probability distribution (FITS file)
        """
    )
    parser.add_argument('output', nargs="?",
        help="""
            Output catalog (FITS file)
            If omitted, it will be "INPUT.cat.fits"
        """
    )

    args = parser.parse_args()
    main(**vars(args))


def main(input, output=None):
    if output is None:
        root, ext = os.path.splitext(input)
        output = root + ".cat.fits"

    object_id, pdf, zmax = hsc_stdio.load_pdf(input)
    hsc_stdio.save_catalog(output, object_id, pdf, zmax)


if __name__ == "__main__":
    startup()
