#!/bin/echo This is not executable but a library:

# Copyright (c) 2016-2017 Sogo Mineo
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import numpy
import tensorflow as tf

import itertools, json, math


class PdfGuesser(object):
    def __init__(self, model_path):
        #
        # Load the model parameter
        #
        model = json.load(open(model_path, "r"))

        layers     = model["layers"    ]
        activation = model["activation"]
        xscale     = model["xscale"    ]
        zmax       = model["zmax"      ]
        nbins      = model["nbins"     ]
        weights    = model["weights"   ]
        biases     = model["biases"    ]

        # Save normalization
        self.xscale = numpy.asarray(xscale , dtype=numpy.float64)
        self.zmax   = zmax
        self.nbins  = nbins

        #
        # Create the neural net
        #
        weights    = [ numpy.asarray(W, dtype=numpy.float32) for W in weights ]
        biases     = [ numpy.asarray(b, dtype=numpy.float32) for b in biases ]

        act_funcs = {"softplus": tf.nn.softplus, "relu": tf.nn.relu, };

        sess = tf.InteractiveSession()
        keep_prob = tf.placeholder(tf.float32)

        def new_layers(input, W, b, is_last=False):
            h = tf.matmul(input, W) + b
            if is_last:
                # h = tf.nn.softmax(h)
                pass
            else:
                h = act_funcs[activation](h)

            return h

        dim = weights[0].shape[0]
        x  = tf.placeholder(tf.float32, shape=[None, dim]) # input
        h = x
        i = 0

        for outdim in layers:
            if outdim == "dropout":
                h = tf.nn.dropout(h, keep_prob)
            else:
                h = new_layers(h, weights[i], biases[i], False)
                i += 1

        h = new_layers(h, weights[i], biases[i], True)
        i += 1
        assert i == len(biases)

        pdf = tf.nn.softmax(h)

        # save neural net
        self.sess      = sess
        self.keep_prob = keep_prob
        self.x         = x
        self.pdf       = pdf

    def __call__(self, xdata):
        xdata = xdata * self.xscale
        numpy.arcsinh(xdata, out=xdata)

        xdata = xdata.astype(numpy.float32)

        return self.sess.run(self.pdf, feed_dict={
            self.x: xdata,
            self.keep_prob: 1.0
        })
