#!/usr/bin/env python

import numpy
import re, sys

def main():
    npz_in, out_prefix = sys.argv[1:]

    catalogs = numpy.load(npz_in)

    if out_prefix and not out_prefix.endswith("-"):
        out_prefix += "-"

    for set_name in ["train_set", "validate_set", "test_set"]:
        textify(out_prefix + set_name + ".txt", catalogs[set_name])

def textify(file_path, catalog):
    format_dict = {
        numpy.dtype(int): "%20d",
        numpy.dtype(float): "%-15.8e", # Assume that values actually have only the precision of float32
    }
    header_format_dict = dict(
        (key, "%{}s".format(re.match(r'%([#0 +-]*[0-9]+)', val).group(1)))
        for key, val in format_dict.iteritems()
    )

    columns = [ catalog[key] for key in catalog.dtype.names ]
    format = ' '.join(format_dict[col.dtype] for col in columns)
    header = ' '.join(header_format_dict[col.dtype] for col in columns) % catalog.dtype.names

    with open(file_path, "w") as fout:
        print >> fout, '#' + header[1:]
        for tuple in zip(*columns):
            print >> fout, format % tuple


if __name__ == "__main__":
    main()
